@@ -877,7 +877,7 @@ ptp_read_func(
             xread--;
         }
 
-        int putfunc_ret = handler->putfunc(NULL, handler->priv, xread, bytes, &written);
+        int putfunc_ret = handler->putfunc(NULL, handler->priv, xread, bytes);
         LIBMTP_USB_DEBUG("handler->putfunc ret = 0x%x\n", putfunc_ret);
         if (putfunc_ret != PTP_RC_OK)
             return putfunc_ret;
@@ -1085,8 +1085,7 @@ memory_getfunc(PTPParams* params, void* private,
 
 static uint16_t
 memory_putfunc(PTPParams* params, void* private,
-        unsigned long sendlen, unsigned char *data,
-        unsigned long *putlen
+        unsigned long sendlen, unsigned char *data
         ) {
     PTPMemHandlerPrivate* priv = (PTPMemHandlerPrivate*) private;
 
@@ -1096,7 +1095,6 @@ memory_putfunc(PTPParams* params, void* private,
     }
     memcpy(priv->data + priv->curoff, data, sendlen);
     priv->curoff += sendlen;
-    *putlen = sendlen;
     return PTP_RC_OK;
 }
 
@@ -1159,7 +1157,7 @@ ptp_exit_recv_memory_handler(PTPDataHandler *handler,
 /* send / receive functions */
 
 uint16_t
-ptp_usb_sendreq(PTPParams* params, PTPContainer* req) {
+ptp_usb_sendreq(PTPParams* params, PTPContainer* req, int dataphase) {
     uint16_t ret;
     PTPUSBBulkContainer usbreq;
     PTPDataHandler memhandler;
@@ -1168,7 +1166,7 @@ ptp_usb_sendreq(PTPParams* params, PTPContainer* req) {
 
     char txt[256];
 
-    (void) ptp_render_opcode(params, req->Code, sizeof (txt), txt);
+    (void) ptp_render_ofc(params, req->Code, sizeof (txt), txt);
     LIBMTP_USB_DEBUG("REQUEST: 0x%04x, %s\n", req->Code, txt);
 
     /* build appropriate USB container */
@@ -1351,12 +1349,11 @@ ptp_usb_getdata(PTPParams* params, PTPContainer* ptp, PTPDataHandler *handler) {
                 break;
             }
         }
-        if (rlen == PTP_USB_BULK_HS_MAX_PACKET_LEN_READ) {
+        if (rlen == ptp_usb->inep_maxpacket) {
             /* Copy first part of data to 'data' */
             putfunc_ret =
                     handler->putfunc(
-                    params, handler->priv, rlen - PTP_USB_BULK_HDR_LEN, usbdata.payload.data,
-                    &written
+                    params, handler->priv, rlen - PTP_USB_BULK_HDR_LEN, usbdata.payload.data
                     );
             if (putfunc_ret != PTP_RC_OK)
                 return putfunc_ret;
@@ -1426,14 +1423,13 @@ ptp_usb_getdata(PTPParams* params, PTPContainer* ptp, PTPDataHandler *handler) {
         putfunc_ret =
                 handler->putfunc(
                 params, handler->priv, rlen - PTP_USB_BULK_HDR_LEN,
-                usbdata.payload.data,
-                &written
+                usbdata.payload.data
                 );
         if (putfunc_ret != PTP_RC_OK)
             return putfunc_ret;
 
         if (FLAG_NO_ZERO_READS(ptp_usb) &&
-                len + PTP_USB_BULK_HDR_LEN == PTP_USB_BULK_HS_MAX_PACKET_LEN_READ) {
+                len + PTP_USB_BULK_HDR_LEN == ptp_usb->inep_maxpacket) {
 
             LIBMTP_USB_DEBUG("Reading in extra terminating byte\n");
 
@@ -1459,8 +1455,8 @@ ptp_usb_getdata(PTPParams* params, PTPContainer* ptp, PTPDataHandler *handler) {
             xread = bulk.result.transferred_bytes;
 
             if (result != 1)
-                LIBMTP_INFO("Could not read in extra byte for PTP_USB_BULK_HS_MAX_PACKET_LEN_READ long file, return value 0x%04x\n", result);
-        } else if (len + PTP_USB_BULK_HDR_LEN == PTP_USB_BULK_HS_MAX_PACKET_LEN_READ && params->split_header_data == 0) {
+                LIBMTP_INFO("Could not read in extra byte for %d bytes long file, return value 0x%04x\n", ptp_usb->inep_maxpacket, result);
+        } else if (len + PTP_USB_BULK_HDR_LEN == ptp_usb->inep_maxpacket && params->split_header_data == 0) {
             int zeroresult = 0, xread;
             unsigned char zerobyte = 0;
 
@@ -1695,6 +1691,17 @@ ptp_usb_event_wait(PTPParams* params, PTPContainer* event) {
     return ptp_usb_event(params, event, PTP_EVENT_CHECK);
 }
 
+uint16_t
+ptp_usb_event_async (PTPParams* params, PTPEventCbFn cb, void *user_data) {
+	/* Unsupported */
+	return PTP_ERROR_CANCEL;
+}
+
+int LIBMTP_Handle_Events_Timeout_Completed(struct timeval *tv, int *completed) {
+	/* Unsupported */
+	return -12;
+}
+
 uint16_t
 ptp_usb_control_cancel_request(PTPParams *params, uint32_t transactionid) {
     PTP_USB *ptp_usb = (PTP_USB *) (params->data);
