@@ -77,6 +77,12 @@ struct mtpdevice_list_struct {
 };
 typedef struct mtpdevice_list_struct mtpdevice_list_t;
 
+struct ptp_event_cb_data {
+  PTPEventCbFn cb;
+  void *user_data;
+  PTPParams *params;
+};
+
 static const LIBMTP_device_entry_t mtp_device_table[] = {
 /* We include an .h file which is shared between us and libgphoto2 */
 #include "music-players.h"
@@ -593,13 +599,11 @@ int LIBMTP_Check_Specific_Device(int busno, int devno)
 
   nrofdevs = libusb_get_device_list (NULL, &devs);
   for (i = 0; i < nrofdevs ; i++ ) {
-
     if (libusb_get_bus_number(devs[i]) != busno)
-      continue;
+	continue;
     if (libusb_get_device_address(devs[i]) != devno)
-      continue;
-
-      if (probe_device_descriptor(devs[i], NULL))
+	continue;
+    if (probe_device_descriptor(devs[i], NULL))
 	return 1;
   }
   return 0;
@@ -835,7 +839,6 @@ ptp_read_func (
   int ret = 0;
   int xread;
   unsigned long curread = 0;
-  unsigned long written;
   unsigned char *bytes;
   int expect_terminator_byte = 0;
   unsigned long usb_inep_maxpacket_size;
@@ -916,7 +919,7 @@ ptp_read_func (
       xread--;
     }
 
-    int putfunc_ret = handler->putfunc(NULL, handler->priv, xread, bytes, &written);
+    int putfunc_ret = handler->putfunc(NULL, handler->priv, xread, bytes);
     if (putfunc_ret != PTP_RC_OK)
       return putfunc_ret;
 
@@ -1099,8 +1102,7 @@ memory_getfunc(PTPParams* params, void* private,
 
 static uint16_t
 memory_putfunc(PTPParams* params, void* private,
-	       unsigned long sendlen, unsigned char *data,
-	       unsigned long *putlen
+	       unsigned long sendlen, unsigned char *data
 ) {
 	PTPMemHandlerPrivate* priv = (PTPMemHandlerPrivate*)private;
 
@@ -1110,7 +1112,6 @@ memory_putfunc(PTPParams* params, void* private,
 	}
 	memcpy (priv->data + priv->curoff, data, sendlen);
 	priv->curoff += sendlen;
-	*putlen = sendlen;
 	return PTP_RC_OK;
 }
 
@@ -1172,7 +1173,7 @@ ptp_exit_recv_memory_handler (PTPDataHandler *handler,
 /* send / receive functions */
 
 uint16_t
-ptp_usb_sendreq (PTPParams* params, PTPContainer* req)
+ptp_usb_sendreq (PTPParams* params, PTPContainer* req, int dataphase)
 {
 	uint16_t ret;
 	PTPUSBBulkContainer usbreq;
@@ -1182,7 +1183,7 @@ ptp_usb_sendreq (PTPParams* params, PTPContainer* req)
 
 	char txt[256];
 
-	(void) ptp_render_opcode (params, req->Code, sizeof(txt), txt);
+	(void) ptp_render_ofc (params, req->Code, sizeof(txt), txt);
 	LIBMTP_USB_DEBUG("REQUEST: 0x%04x, %s\n", req->Code, txt);
 
 	/* build appropriate USB container */
@@ -1320,7 +1321,6 @@ ptp_usb_getdata (PTPParams* params, PTPContainer* ptp, PTPDataHandler *handler)
 {
 	uint16_t ret;
 	PTPUSBBulkContainer usbdata;
-	unsigned long	written;
 	PTP_USB *ptp_usb = (PTP_USB *) params->data;
 	int putfunc_ret;
 
@@ -1362,12 +1362,11 @@ ptp_usb_getdata (PTPParams* params, PTPContainer* ptp, PTPDataHandler *handler)
 				break;
 			}
 		}
-		if (rlen == PTP_USB_BULK_HS_MAX_PACKET_LEN_READ) {
+		if (rlen == ptp_usb->inep_maxpacket) {
 		  /* Copy first part of data to 'data' */
 		  putfunc_ret =
 		    handler->putfunc(
-				     params, handler->priv, rlen - PTP_USB_BULK_HDR_LEN, usbdata.payload.data,
-				     &written
+				     params, handler->priv, rlen - PTP_USB_BULK_HDR_LEN, usbdata.payload.data
 				     );
 		  if (putfunc_ret != PTP_RC_OK)
 		    return putfunc_ret;
@@ -1437,14 +1436,13 @@ ptp_usb_getdata (PTPParams* params, PTPContainer* ptp, PTPDataHandler *handler)
 		putfunc_ret =
 		  handler->putfunc(
 				   params, handler->priv, rlen - PTP_USB_BULK_HDR_LEN,
-				   usbdata.payload.data,
-				   &written
+				   usbdata.payload.data
 				   );
 		if (putfunc_ret != PTP_RC_OK)
 		  return putfunc_ret;
 
 		if (FLAG_NO_ZERO_READS(ptp_usb) &&
-		    len+PTP_USB_BULK_HDR_LEN == PTP_USB_BULK_HS_MAX_PACKET_LEN_READ) {
+		    len+PTP_USB_BULK_HDR_LEN == ptp_usb->inep_maxpacket) {
 
 		  LIBMTP_USB_DEBUG("Reading in extra terminating byte\n");
 
@@ -1459,8 +1457,8 @@ ptp_usb_getdata (PTPParams* params, PTPContainer* ptp, PTPDataHandler *handler)
 					 ptp_usb->timeout);
 
 		  if (result != 1)
-		    LIBMTP_INFO("Could not read in extra byte for PTP_USB_BULK_HS_MAX_PACKET_LEN_READ long file, return value 0x%04x\n", result);
-		} else if (len+PTP_USB_BULK_HDR_LEN == PTP_USB_BULK_HS_MAX_PACKET_LEN_READ && params->split_header_data == 0) {
+		    LIBMTP_INFO("Could not read in extra byte for %d byte long file, return value 0x%04x\n", ptp_usb->inep_maxpacket, result);
+		} else if (len+PTP_USB_BULK_HDR_LEN == ptp_usb->inep_maxpacket && params->split_header_data == 0) {
 		  int zeroresult = 0, xread;
 		  unsigned char zerobyte = 0;
 
@@ -1651,6 +1649,109 @@ ptp_usb_event_wait (PTPParams* params, PTPContainer* event) {
 	return ptp_usb_event (params, event, PTP_EVENT_CHECK);
 }
 
+static void
+ptp_usb_event_cb (struct libusb_transfer *t) {
+	struct ptp_event_cb_data *data = t->user_data;
+	PTPParams *params = data->params;
+	PTPUSBEventContainer *usbevent = (void *)t->buffer;
+	PTPContainer event = {0,};
+	uint16_t code;
+
+	switch (t->status) {
+	case LIBUSB_TRANSFER_COMPLETED:
+		if (t->actual_length < 8) {
+			libusb_glue_error (params,
+				"PTP: reading event an short read of %ld bytes occurred\n",
+				t->actual_length);
+			code = PTP_ERROR_IO;
+		} else {
+			event.Code=dtoh16(usbevent->code);
+			event.SessionID=params->session_id;
+			event.Transaction_ID=dtoh32(usbevent->trans_id);
+			event.Param1=dtoh32(usbevent->param1);
+			event.Param2=dtoh32(usbevent->param2);
+			event.Param3=dtoh32(usbevent->param3);
+			code = PTP_RC_OK;
+		}
+		break;
+	case LIBUSB_TRANSFER_TIMED_OUT:
+		code = PTP_ERROR_TIMEOUT;
+		break;
+	case LIBUSB_TRANSFER_CANCELLED:
+		code = PTP_ERROR_CANCEL;
+		break;
+	case LIBUSB_TRANSFER_STALL:
+		code = PTP_ERROR_DATA_EXPECTED;
+		break;
+	case LIBUSB_TRANSFER_ERROR:
+	case LIBUSB_TRANSFER_NO_DEVICE:
+	case LIBUSB_TRANSFER_OVERFLOW:
+	default:
+		code = PTP_ERROR_IO;
+		break;
+	}
+	if (code != PTP_RC_OK) {
+		libusb_glue_error (params,
+			"PTP: reading event an error 0x%02x occurred\n",
+			t->status);
+	}
+	data->cb(params, code, &event, data->user_data);
+	free(data);
+}
+
+uint16_t
+ptp_usb_event_async (PTPParams* params, PTPEventCbFn cb, void *user_data) {
+	PTP_USB *ptp_usb;
+	PTPUSBEventContainer *usbevent;
+	struct ptp_event_cb_data *data;
+	struct libusb_transfer *t;
+	int ret;
+
+	if (params == NULL) {
+		return PTP_ERROR_BADPARAM;
+	}
+
+        usbevent = calloc(1, sizeof(*usbevent));
+        if (usbevent == NULL) {
+		return PTP_ERROR_IO;
+        }
+
+	data = malloc(sizeof(*data));
+	if (data == NULL) {
+		free(usbevent);
+		return PTP_ERROR_IO;
+	}
+
+	t = libusb_alloc_transfer(0);
+	if (t == NULL) {
+		free(data);
+		free(usbevent);
+		return PTP_ERROR_IO;
+	}
+
+	data->cb = cb;
+	data->user_data = user_data;
+	data->params = params;
+
+	ptp_usb = (PTP_USB *)(params->data);
+	libusb_fill_interrupt_transfer(t, ptp_usb->handle, ptp_usb->intep,
+	                               (unsigned char *)usbevent, sizeof(*usbevent),
+	                               ptp_usb_event_cb, data, 0);
+	t->flags = LIBUSB_TRANSFER_FREE_BUFFER | LIBUSB_TRANSFER_FREE_TRANSFER;
+
+	ret = libusb_submit_transfer(t);
+	return ret == 0 ? PTP_RC_OK : PTP_ERROR_IO;
+}
+
+/**
+ * Trivial wrapper around the most generic libusb method for polling for events.
+ * Can be used to drive asynchronous event detection.
+ */
+int LIBMTP_Handle_Events_Timeout_Completed(struct timeval *tv, int *completed) {
+	/* Pass NULL for context as libmtp always uses the default context */
+	return libusb_handle_events_timeout_completed(NULL, tv, completed);
+}
+
 uint16_t
 ptp_usb_control_cancel_request (PTPParams *params, uint32_t transactionid) {
 	PTP_USB *ptp_usb = (PTP_USB *)(params->data);
